@@ -11,7 +11,6 @@ import (
 	"path/filepath"
 	"sort"
 	"strings"
-	"sync"
 	"time"
 
 	"github.com/pborman/uuid"
@@ -44,6 +43,7 @@ type PublishedRepo struct {
 	NotAutomatic         string
 	ButAutomaticUpgrades string
 	Label                string
+	Suite                string
 	// Architectures is a list of all architectures published
 	Architectures []string
 	// SourceKind is "local"/"repo"
@@ -62,6 +62,9 @@ type PublishedRepo struct {
 	// Skip contents generation
 	SkipContents bool
 
+	// Skip bz2 compression for index files
+	SkipBz2 bool
+
 	// True if repo is being re-published
 	rePublishing bool
 
@@ -281,7 +284,7 @@ func NewPublishedRepo(storage, prefix, distribution string, architectures []stri
 	return result, nil
 }
 
-// MarshalJSON requires object to be "loeaded completely"
+// MarshalJSON requires object to be "loaded completely"
 func (p *PublishedRepo) MarshalJSON() ([]byte, error) {
 	type sourceInfo struct {
 		Component, Name string
@@ -308,9 +311,11 @@ func (p *PublishedRepo) MarshalJSON() ([]byte, error) {
 		"Distribution":         p.Distribution,
 		"Label":                p.Label,
 		"Origin":               p.Origin,
+		"Suite":                p.Suite,
 		"NotAutomatic":         p.NotAutomatic,
 		"ButAutomaticUpgrades": p.ButAutomaticUpgrades,
 		"Prefix":               p.Prefix,
+		"Path":                 p.GetPath(),
 		"SourceKind":           p.SourceKind,
 		"Sources":              sources,
 		"Storage":              p.Storage,
@@ -357,6 +362,10 @@ func (p *PublishedRepo) String() string {
 		extras = append(extras, fmt.Sprintf("label: %s", p.Label))
 	}
 
+	if p.Suite != "" {
+		extras = append(extras, fmt.Sprintf("suite: %s", p.Suite))
+	}
+
 	extra = strings.Join(extras, ", ")
 
 	if extra != "" {
@@ -485,6 +494,25 @@ func (p *PublishedRepo) GetLabel() string {
 	return p.Label
 }
 
+// GetPath returns the unique name of the repo
+func (p *PublishedRepo) GetPath() string {
+	prefix := p.StoragePrefix()
+
+	if prefix == "" {
+		return p.Distribution
+	}
+
+	return fmt.Sprintf("%s/%s", prefix, p.Distribution)
+}
+
+// GetSuite returns default or manual Suite:
+func (p *PublishedRepo) GetSuite() string {
+	if p.Suite == "" {
+		return p.Distribution
+	}
+	return p.Suite
+}
+
 // Publish publishes snapshot (repository) contents, links package files, generates Packages & Release files, signs them
 func (p *PublishedRepo) Publish(packagePool aptly.PackagePool, publishedStorageProvider aptly.PublishedStorageProvider,
 	collectionFactory *CollectionFactory, signer pgp.Signer, progress aptly.Progress, forceOverwrite bool) error {
@@ -560,9 +588,17 @@ func (p *PublishedRepo) Publish(packagePool aptly.PackagePool, publishedStorageP
 	}
 	defer os.RemoveAll(tempDir)
 
-	indexes := newIndexFiles(publishedStorage, basePath, tempDir, suffix, p.AcquireByHash)
+	indexes := newIndexFiles(publishedStorage, basePath, tempDir, suffix, p.AcquireByHash, p.SkipBz2)
 
 	legacyContentIndexes := map[string]*ContentsIndex{}
+	var count int64
+	for _, list := range lists {
+		count = count + int64(list.Len())
+	}
+
+	if progress != nil {
+		progress.InitBar(count, false, aptly.BarPublishGeneratePackageFiles)
+	}
 
 	for component, list := range lists {
 		hadUdebs := false
@@ -572,10 +608,6 @@ func (p *PublishedRepo) Publish(packagePool aptly.PackagePool, publishedStorageP
 			indexes.PackageIndex(component, arch, false, false)
 		}
 
-		if progress != nil {
-			progress.InitBar(int64(list.Len()), false)
-		}
-
 		list.PrepareIndex()
 
 		contentIndexes := map[string]*ContentsIndex{}
@@ -612,8 +644,7 @@ func (p *PublishedRepo) Publish(packagePool aptly.PackagePool, publishedStorageP
 			// to push each path of the package into the database.
 			// We'll want this batched so as to avoid an excessive
 			// amount of write() calls.
-			tempDB.StartBatch()
-			defer tempDB.FinishBatch()
+			batch := tempDB.CreateBatch()
 
 			for _, arch := range p.Architectures {
 				if pkg.MatchesArchitecture(arch) {
@@ -632,7 +663,7 @@ func (p *PublishedRepo) Publish(packagePool aptly.PackagePool, publishedStorageP
 								contentIndexesMap[key] = contentIndex
 							}
 
-							contentIndex.Push(qualifiedName, contents)
+							contentIndex.Push(qualifiedName, contents, batch)
 						}
 					}
 
@@ -657,7 +688,7 @@ func (p *PublishedRepo) Publish(packagePool aptly.PackagePool, publishedStorageP
 			pkg.extra = nil
 			pkg.contents = nil
 
-			return nil
+			return batch.Write()
 		})
 
 		if err != nil {
@@ -684,10 +715,6 @@ func (p *PublishedRepo) Publish(packagePool aptly.PackagePool, publishedStorageP
 			}
 		}
 
-		if progress != nil {
-			progress.ShutdownBar()
-		}
-
 		udebs := []bool{false}
 		if hadUdebs {
 			udebs = append(udebs, true)
@@ -707,6 +734,7 @@ func (p *PublishedRepo) Publish(packagePool aptly.PackagePool, publishedStorageP
 				release["Component"] = component
 				release["Origin"] = p.GetOrigin()
 				release["Label"] = p.GetLabel()
+				release["Suite"] = p.GetSuite()
 				if p.AcquireByHash {
 					release["Acquire-By-Hash"] = "yes"
 				}
@@ -746,6 +774,7 @@ func (p *PublishedRepo) Publish(packagePool aptly.PackagePool, publishedStorageP
 	}
 
 	if progress != nil {
+		progress.ShutdownBar()
 		progress.Printf("Finalizing metadata files...\n")
 	}
 
@@ -763,7 +792,7 @@ func (p *PublishedRepo) Publish(packagePool aptly.PackagePool, publishedStorageP
 		release["ButAutomaticUpgrades"] = p.ButAutomaticUpgrades
 	}
 	release["Label"] = p.GetLabel()
-	release["Suite"] = p.Distribution
+	release["Suite"] = p.GetSuite()
 	release["Codename"] = p.Distribution
 	release["Date"] = time.Now().UTC().Format("Mon, 2 Jan 2006 15:04:05 MST")
 	release["Architectures"] = strings.Join(utils.StrSlicesSubstract(p.Architectures, []string{ArchitectureSource}), " ")
@@ -852,7 +881,6 @@ func (p *PublishedRepo) RemoveFiles(publishedStorageProvider aptly.PublishedStor
 
 // PublishedRepoCollection does listing, updating/adding/deleting of PublishedRepos
 type PublishedRepoCollection struct {
-	*sync.RWMutex
 	db   database.Storage
 	list []*PublishedRepo
 }
@@ -860,8 +888,7 @@ type PublishedRepoCollection struct {
 // NewPublishedRepoCollection loads PublishedRepos from DB and makes up collection
 func NewPublishedRepoCollection(db database.Storage) *PublishedRepoCollection {
 	return &PublishedRepoCollection{
-		RWMutex: &sync.RWMutex{},
-		db:      db,
+		db: db,
 	}
 }
 
@@ -914,21 +941,16 @@ func (collection *PublishedRepoCollection) CheckDuplicate(repo *PublishedRepo) *
 }
 
 // Update stores updated information about repo in DB
-func (collection *PublishedRepoCollection) Update(repo *PublishedRepo) (err error) {
-	err = collection.db.Put(repo.Key(), repo.Encode())
-	if err != nil {
-		return
-	}
+func (collection *PublishedRepoCollection) Update(repo *PublishedRepo) error {
+	batch := collection.db.CreateBatch()
+	batch.Put(repo.Key(), repo.Encode())
 
 	if repo.SourceKind == SourceLocalRepo {
 		for component, item := range repo.sourceItems {
-			err = collection.db.Put(repo.RefKey(component), item.packageRefs.Encode())
-			if err != nil {
-				return
-			}
+			batch.Put(repo.RefKey(component), item.packageRefs.Encode())
 		}
 	}
-	return
+	return batch.Write()
 }
 
 // LoadComplete loads additional information for remote repo
@@ -1170,6 +1192,7 @@ func (collection *PublishedRepoCollection) Remove(publishedStorageProvider aptly
 	storage, prefix, distribution string, collectionFactory *CollectionFactory, progress aptly.Progress,
 	force, skipCleanup bool) error {
 
+	// TODO: load via transaction
 	collection.loadList()
 
 	repo, err := collection.ByStoragePrefixDistribution(storage, prefix, distribution)
@@ -1221,17 +1244,12 @@ func (collection *PublishedRepoCollection) Remove(publishedStorageProvider aptly
 		}
 	}
 
-	err = collection.db.Delete(repo.Key())
-	if err != nil {
-		return err
-	}
+	batch := collection.db.CreateBatch()
+	batch.Delete(repo.Key())
 
 	for _, component := range repo.Components() {
-		err = collection.db.Delete(repo.RefKey(component))
-		if err != nil {
-			return err
-		}
+		batch.Delete(repo.RefKey(component))
 	}
 
-	return nil
+	return batch.Write()
 }
